@@ -23,6 +23,7 @@
 
 #include <stdio.h>
 #include <sys/types.h>
+#include <inttypes.h> /* for PRI[du]64 */
 #include "config.h"
 #include <math.h>
 
@@ -45,11 +46,15 @@
    subdomains whose disjoint union is the total domain
    (arXiv:1303.6193).
  */
-void omega_d_precalc_(/* INPUTS */
-                      int64_t *scal_av_n_gridsize,
+void Omega_D_precalc(/* INPUTS */
+                      int64_t *scal_av_n_gridsize1,
+                      int64_t *scal_av_n_gridsize2,
+                      int64_t *scal_av_n_gridsize3,
                       double **I_inv,
                       double **II_inv,
                       double **III_inv,
+                      /* aD_norm: proportional to initial volumes^(1/3) */
+                      double **aD_norm,
                       int64_t *n_t_inhomog,
                       double *a_FLRW_init,
                       double *H1bg,
@@ -65,18 +70,47 @@ void omega_d_precalc_(/* INPUTS */
   int want_planar = 0;
   int want_spherical = 0;
 
-  /* PRECALC_PRINT_ALL should only be turned on for testing purposes
-     or small enough simulations; it is not intended for normal use: */
+  /*! PRECALC_PRINT_ALL should only be turned on for small simulations
+     or testing purposes; it is not intended for big simulations: */
   /* #define PRECALC_PRINT_ALL 1 */
 
+#ifdef PRECALC_PRINT_ALL
+  /*! If OMEGA_GLOBAL_NORMALISE is turned on, then the Omegas are
+     normalised to the global H_D (as in Roukema+2013 JCAP 10 043,
+     arXiv:1303.4444) instead of the local H_D. This avoids getting
+     meaningless Omega's near turnaround when the local H_D approaches
+     zero.  COMMENT: This is only relevant if PRECALC_PRINT_ALL is
+     enabled.
+  */
+#define OMEGA_GLOBAL_NORMALISE 1
+#endif
+
   /* #define N_PLOT_T 32 */
   /* #define N_PLOT_SUBDOM 512 */
   /* #define N_PLOT_SUBDOM 4096 */
   /* #define N_PLOT_SUBDOM 64 */
 
   uint64_t n_subdom;
-  unsigned int n_t_inhomog_u;
-#define N_COSM_I 1
+  uint64_t n_t_inhomog_u;
+
+  /* define initial and final N_COSM values:
+     1 1 gives the EdS case ("1" = true EdS)
+     0 0 gives LCDM (Planck 2015 normalised)
+     0 1 gives both (outputs would have to be analysed separately)
+  */
+#if defined INHOM_LCDM
+ #define N_COSM_A 0
+ #define N_COSM_B 0
+#elif defined INHOM_EDS
+ #define N_COSM_A 1
+ #define N_COSM_B 1
+#elif defined INHOM_LCDM_EDS /* both LCDM and EdS */
+ #define N_COSM_A 0
+ #define N_COSM_B 1
+#else /* default is EdS */
+ #define N_COSM_A 1
+ #define N_COSM_B 1
+#endif
   /* Although N_COSM is small, it is used in allocation
      routines, which require 64-bit integers for sizes */
 #define N_COSM ((uint64_t)2)
@@ -87,6 +121,7 @@ void omega_d_precalc_(/* INPUTS */
   double t_i[N_COSM];
   double t_0[N_COSM];
   double **t_background; /* [N_COSM][N_PLOT_T]; */
+  double **a_background; /* [N_COSM][N_PLOT_T]; */
   double ***a_D; /* [N_COSM][N_PLOT_SUBDOM][N_PLOT_T] */
   double *a_D_sortable;
   double ***dot_a_D;
@@ -96,6 +131,9 @@ void omega_d_precalc_(/* INPUTS */
   double ***rza_R_D;
   double *rza_Q_D_sortable;
   double *rza_R_D_sortable;
+#ifndef HAVE_GSL_SORT2
+  uint64_t *i_QR_index, i_subdom_sorted; /* sorting indices for old GSL */
+#endif
   double *a_D_3_sortable_Q, *a_D_3_sortable_R;
   double a_D_3_total; /* for volume-weighted means */
   double a_D_3_half, a_D_3_cumul;  /* for volume-weighted medians */
@@ -124,6 +162,10 @@ void omega_d_precalc_(/* INPUTS */
   double **OmR_D_global; /* [N_COSM][N_PLOT_T]; */
   double **OmQ_D_median; /* [N_COSM][N_PLOT_T]; */
   double **OmR_D_median; /* [N_COSM][N_PLOT_T]; */
+#ifdef OMEGA_GLOBAL_NORMALISE
+  double HDlocal_sq_over_HDglobal_sq; /* (H_D/H_D_global)^2 */
+#endif
+
   double **a_D_voids; /* [N_COSM][N_PLOT_T]; */ /* global a_D for expanding domains */
   long int i_voids;
   long int i_uncollapsed;
@@ -168,18 +210,20 @@ void omega_d_precalc_(/* INPUTS */
   r_gsl = gsl_rng_alloc (T_gsl); /* this gets reallocated each time the
                                     function gets called */
 
-  n_t_inhomog_u = (unsigned)(*n_t_inhomog);
+  n_t_inhomog_u = (uint64_t)(*n_t_inhomog);
 
 #ifdef __GNUC__
   /* check allocation */
-  printf("malloc_usable_size(*I_inv) = %lu malloc_usable_size(*II_inv) = %lu  (*scal_av_n_gridsize) = %ld\n",
-         malloc_usable_size(*I_inv), malloc_usable_size(*II_inv), (*scal_av_n_gridsize));
+  printf("malloc_usable_size(*I_inv) = %zu malloc_usable_size(*II_inv) = %zu  (*scal_av_n_gridsize1) = %" PRId64 "\n",
+         malloc_usable_size(*I_inv), malloc_usable_size(*II_inv), (*scal_av_n_gridsize1));
 #endif
 
+  printf("Omega_D_precalc: n_t_inhomog = %p, *n_t_inhomog = %" PRId64 ", n_t_inhomog_u = %" PRIu64 "\n",
+         (void*)n_t_inhomog, *n_t_inhomog, n_t_inhomog_u);
 
-  n_subdom = (uint64_t)(*scal_av_n_gridsize)
-    * (uint64_t)(*scal_av_n_gridsize)
-    * (uint64_t)(*scal_av_n_gridsize);
+  n_subdom = (uint64_t)(*scal_av_n_gridsize1)
+    * (uint64_t)(*scal_av_n_gridsize2)
+    * (uint64_t)(*scal_av_n_gridsize3);
 
 
   alloc_failed += alloc_big_array_3D_d(N_COSM,n_subdom,n_t_inhomog_u,
@@ -196,8 +240,8 @@ void omega_d_precalc_(/* INPUTS */
                        &a_D_3_sortable_R, 1);
   alloc_failed += alloc_big_array_3D_d(N_COSM,n_subdom,n_t_inhomog_u,
                        &dot_a_D, 1);
-  i_t_collapse = malloc(n_subdom*sizeof(int));
-  collapsed = malloc(n_subdom*sizeof(int));
+  i_t_collapse = malloc((size_t)(n_subdom*sizeof(int)));
+  collapsed = malloc((size_t)(n_subdom*sizeof(int)));
 
   alloc_failed += alloc_big_array_3D_i(N_COSM,n_subdom,n_t_inhomog_u,
                        &unphysical, 1);
@@ -223,6 +267,8 @@ void omega_d_precalc_(/* INPUTS */
 
   alloc_failed += alloc_big_array_2D_d(N_COSM,n_t_inhomog_u,
                        &t_background, 1);
+  alloc_failed += alloc_big_array_2D_d(N_COSM,n_t_inhomog_u,
+                       &a_background, 1);
   alloc_failed += alloc_big_array_2D_d(N_COSM,n_t_inhomog_u,
                        &a_D_global, 1);
   alloc_failed += alloc_big_array_2D_d(N_COSM,n_t_inhomog_u,
@@ -270,7 +316,7 @@ void omega_d_precalc_(/* INPUTS */
   rza_integrand_params.precalculated_invariants.enabled = 1;
 
 
-  printf("Omega_D_precalc: number of subdomains: %lu\n",n_subdom);
+  printf("Omega_D_precalc: number of subdomains: %" PRIu64 "\n",n_subdom);
 
 
   background_cosm_params.flatFLRW = 1;
@@ -280,7 +326,7 @@ void omega_d_precalc_(/* INPUTS */
   background_cosm_params.inhomog_a_scale_factor_now = 1.0;
 
 
-  for(i_EdS=N_COSM_I; i_EdS<(int)N_COSM; i_EdS++){
+  for(i_EdS=N_COSM_A; i_EdS<(int)N_COSM_B+1; i_EdS++){
     background_cosm_params.EdS = i_EdS;
     printf("background_cosm_params.EdS  = %d\n",
            background_cosm_params.EdS);
@@ -295,8 +341,12 @@ void omega_d_precalc_(/* INPUTS */
       background_cosm_params.OmLam_0 = 0.0;
       background_cosm_params.Ombary_0 = 0.022
         /exp(2.0*log(background_cosm_params.H_0 / 100));
-      /* Planck: Ade et al 2013 arXiv:1303.5076v3, Table 2 */
-      background_cosm_params.sigma8 = 0.83;
+      /*
+         Planck 2015: arXiv:1502.01589, Table 4, final column,
+         adjusted to for CMB-matched EdS; see
+         RMOB17 arXiv:1608.06004)
+      */
+      background_cosm_params.sigma8 = 1.0395;
 
       t_i[i_EdS] = t_EdS(&background_cosm_params,
                          background_cosm_params.inhomog_a_scale_factor_initial,
@@ -308,13 +358,12 @@ void omega_d_precalc_(/* INPUTS */
          background_cosm_params.OmLam_0 = 0.73;
       */
       /*
-         Planck 2015: arXiv:1502.01589
+         Planck 2015: arXiv:1502.01589, Table 4, final column
       */
       background_cosm_params.H_0 = 67.74;
-      background_cosm_params.OmLam_0 = 0.691;
+      background_cosm_params.OmLam_0 = 0.6911;
       background_cosm_params.Ombary_0 = 0.044;
-      /* Planck: Ade et al 2013 arXiv:1303.5076v3, Table 2 */
-      background_cosm_params.sigma8 = 0.83;
+      background_cosm_params.sigma8 = 0.8159;
       /*
         background_cosm_params.H_0 = 50.0;
         background_cosm_params.OmLam_0 = 0.001;
@@ -347,8 +396,8 @@ void omega_d_precalc_(/* INPUTS */
     */
 
 
-    mean_I = gsl_stats_mean( *I_inv, 1, n_subdom);
-    sigma_I = gsl_stats_sd( *I_inv, 1, n_subdom);
+    mean_I = gsl_stats_mean( *I_inv, (size_t)1, (size_t)n_subdom);
+    sigma_I = gsl_stats_sd( *I_inv, (size_t)1, (size_t)n_subdom);
 
 
 #pragma omp parallel                                                    \
@@ -422,14 +471,20 @@ void omega_d_precalc_(/* INPUTS */
                      t_background[i_EdS][i_t],
                      want_verbose);
       };
-
+      /* TODO: remove redundant recalculations of a_background
+         below */
+      a_background[i_EdS][i_t] = a_FLRW_expected;
 
       for(i_subdom=0; i_subdom<n_subdom; i_subdom++){
 
         /* If a line (e.g. of constant y, z) is finished, then
            cumulate the mean linear scale factor and start a new
-           line. */
-        if(i_per_line+1 == *scal_av_n_gridsize){
+           line.
+
+           This option is distracting and should probably be
+           removed.
+        */
+        if(i_per_line+1 == *scal_av_n_gridsize1){
           a_D_linear[i_EdS][i_t] +=
             a_D_linear_per_line/fmax(1.0,a_D_linear_weight);
           i_lines ++;
@@ -441,12 +496,16 @@ void omega_d_precalc_(/* INPUTS */
         };
 
 
-        if( (!collapsed[i_subdom] ||
-             (collapsed[i_subdom] &&
-              i_t_collapse[i_subdom] >= 0 &&
-              i_t <= (unsigned)i_t_collapse[i_subdom]) )
+        if( !collapsed[i_subdom] ||
+             ( collapsed[i_subdom] &&
+               i_t_collapse[i_subdom] >= 0 &&
+               i_t <= (unsigned)i_t_collapse[i_subdom] )
             ){
 
+          /* Initial volume weighting: correct for the case of unequal
+             initial volumes of the domains D */
+          a_D[i_EdS][i_subdom][i_t] *= (*aD_norm)[i_subdom];
+
           /* volume element a_D^3 weightings */
           a_D_3_sortable_Q[i_subdom] = pow(a_D[i_EdS][i_subdom][i_t], 3.0);
           a_D_3_sortable_R[i_subdom] = a_D_3_sortable_Q[i_subdom];
@@ -555,8 +614,8 @@ void omega_d_precalc_(/* INPUTS */
         for(i_subdom=0; i_subdom<n_subdom; i_subdom++){
           Omm_D_1model_1time[i_subdom] = Omm_D[i_EdS][i_subdom][i_t];
         };
-        mean_Omm_D = gsl_stats_mean( Omm_D_1model_1time, 1, n_subdom);
-        sigma_Omm_D = gsl_stats_sd( Omm_D_1model_1time, 1, n_subdom);
+        mean_Omm_D = gsl_stats_mean( Omm_D_1model_1time, (size_t)1, (size_t)n_subdom);
+        sigma_Omm_D = gsl_stats_sd( Omm_D_1model_1time, (size_t)1, (size_t)n_subdom);
         printf("a_FLRW_init, mean_I, mean_Omm_D; EdS-grown-values: %g %g %g\n",
                *a_FLRW_init, mean_I, mean_Omm_D);
         printf("a_FLRW_init, sigma_I, sigma_Omm_D; EdS-grown-values: %g %g %g : %g %g\n",
@@ -581,10 +640,10 @@ void omega_d_precalc_(/* INPUTS */
         a_D_linear[i_EdS][i_t] = -9e99;
       };
 
-      gsl_sort(a_D_sortable, 1, n_subdom); /* sorts in place */
+      gsl_sort(a_D_sortable, (size_t)1, (size_t)n_subdom); /* sorts in place */
       a_D_median[i_EdS][i_t] =
         gsl_stats_median_from_sorted_data(a_D_sortable,
-                                          1, n_subdom);
+                                          (size_t)1, (size_t)n_subdom);
 
       /* Outputs for fortran calling routine. */
       if(1==i_EdS){
@@ -653,8 +712,49 @@ void omega_d_precalc_(/* INPUTS */
 
         /* Sort a_D_3_sortable_Q according to the ascending order of
            rza_Q_D_sortable. */
-        gsl_sort2(rza_Q_D_sortable, 1,
-                  a_D_3_sortable_Q, 1, n_subdom);
+
+#ifdef HAVE_GSL_SORT2
+        gsl_sort2(rza_Q_D_sortable, (size_t)1,
+                  a_D_3_sortable_Q, (size_t)1, (size_t)n_subdom);
+#else
+        /* Old GSL:  */
+        if(NULL == (i_QR_index = malloc(n_subdom*sizeof(uint64_t)))){
+          printf("Omega_D_precalc: Error: out of memory.\n");
+          exit(1);
+        };
+        gsl_sort_index(i_QR_index, rza_Q_D_sortable,
+                       (size_t)1, n_subdom);
+        gsl_sort(rza_Q_D_sortable, (size_t)1, n_subdom);
+        for(i_subdom = 0; i_subdom < n_subdom; i_subdom++){
+          i_subdom_sorted = i_QR_index[i_subdom];
+          if( !collapsed[i_subdom_sorted] ||
+              ( collapsed[i_subdom_sorted] &&
+                i_t_collapse[i_subdom_sorted] >= 0 &&
+                i_t <= (unsigned)i_t_collapse[i_subdom_sorted] )
+              ){
+            a_D_3_sortable_Q[i_subdom] = pow(a_D[i_EdS][i_subdom_sorted][i_t], 3.0);
+          }else{
+            a_D_3_sortable_Q[i_subdom] = 0.0;
+          };
+        };
+        gsl_sort_index(i_QR_index, rza_R_D_sortable,
+                       (size_t)1, n_subdom);
+        gsl_sort(rza_R_D_sortable, (size_t)1, n_subdom);
+        for(i_subdom = 0; i_subdom < n_subdom; i_subdom++){
+          i_subdom_sorted = i_QR_index[i_subdom];
+          if( !collapsed[i_subdom_sorted] ||
+              ( collapsed[i_subdom_sorted] &&
+                i_t_collapse[i_subdom_sorted] >= 0 &&
+                i_t <= (unsigned)i_t_collapse[i_subdom_sorted] )
+              ){
+            a_D_3_sortable_R[i_subdom] = pow(a_D[i_EdS][i_subdom_sorted][i_t], 3.0);
+          }else{
+            a_D_3_sortable_R[i_subdom] = 0.0;
+          };
+        };
+        free(i_QR_index);
+#endif
+
         a_D_3_cumul = 0.0;
         /* Find position of weighted median in array. */
         for(i_subdom = 0;
@@ -670,8 +770,11 @@ void omega_d_precalc_(/* INPUTS */
 
         /* Sort a_D_3_sortable_R according to the ascending order of
            rza_R_D_sortable. */
-        gsl_sort2(rza_R_D_sortable, 1,
-                  a_D_3_sortable_R, 1, n_subdom);
+#ifdef HAVE_GSL_SORT2
+        gsl_sort2(rza_R_D_sortable, (size_t)1,
+                  a_D_3_sortable_R, (size_t)1, (size_t)n_subdom);
+        /* without gsl_sort2: done above */
+#endif
         a_D_3_cumul = 0.0;
         /* Find position of weighted median in array. */
         for(i_subdom = 0;
@@ -693,13 +796,22 @@ void omega_d_precalc_(/* INPUTS */
       };
 
       /* TODO: optionally turn this off for high (*n_t_inhomog). */
-      printf("a_D_global: %e %e : %e %e : %e %e : %e %e %e %e %ld\n",
+      printf("a_D_global: %e %e : %e %e : ",
              t_background[i_EdS][i_t],
              a_FLRW_expected,
              /* : */
              a_D_global[i_EdS][i_t],
-             a_D_voids[i_EdS][i_t],
+             a_D_voids[i_EdS][i_t]
              /* : */
+             );
+      /* bidomain case */
+      if(2==n_subdom){
+        printf(" %e %e : ",
+               a_D[i_EdS][0][i_t],
+               a_D[i_EdS][1][i_t]);
+      };
+
+      printf("%e %e : %e %e %e %e %ld\n",
              dot_a_D_global[i_EdS][i_t],
              H_D_global[i_EdS][i_t] *KMS_PER_MPCGYR,
              /* : */
@@ -735,18 +847,32 @@ void omega_d_precalc_(/* INPUTS */
         (void)a_dot_FLRW_expected; /* avoid compile warning */
 
 #ifdef PRECALC_PRINT_ALL
-        printf(/*nsig=%4.1f%4.1f%4.1f*/" tB2 R : a_D H_D : Omm_D OmQ_D OmR_D  %g %d : %g %g : %g %g %g\n",
+#ifdef OMEGA_GLOBAL_NORMALISE
+        HDlocal_sq_over_HDglobal_sq =
+          pow(H_D[i_EdS][i_subdom][i_t]/H_D_global[i_EdS][i_t], 2.0);
+#endif
+        printf(/*nsig=%4.1f%4.1f%4.1f*/
+               " tB2 z Heff i : a_D H_D : Omm_D OmQ_D OmR_D  %g %g %g %" PRIu64 " : %g %g : %g %g %g\n",
                /* n_sigma[0],
                   n_sigma[1],
                   n_sigma[2],*/
                t_background[i_EdS][i_t],
+               1.0/a_background[i_EdS][i_t]-1.0,
+               H_D_global[i_EdS][i_t]/COSM_H_0_INV_GYR,
                i_subdom,
                a_D[i_EdS][i_subdom][i_t],
                H_D[i_EdS][i_subdom][i_t]/COSM_H_0_INV_GYR,
+#ifdef OMEGA_GLOBAL_NORMALISE
+               Omm_D[i_EdS][i_subdom][i_t] *HDlocal_sq_over_HDglobal_sq,
+               OmQ_D[i_EdS][i_subdom][i_t] *HDlocal_sq_over_HDglobal_sq,
+               /* OmLam_D[i_EdS][i_subdom][i_t], */
+               OmR_D[i_EdS][i_subdom][i_t] *HDlocal_sq_over_HDglobal_sq
+#else
                Omm_D[i_EdS][i_subdom][i_t],
                OmQ_D[i_EdS][i_subdom][i_t],
                /* OmLam_D[i_EdS][i_subdom][i_t], */
                OmR_D[i_EdS][i_subdom][i_t]
+#endif
                );
 #endif
       }; /* for(i_t=0; i_t<(*n_t_inhomog); i_t++) */
@@ -765,6 +891,7 @@ void omega_d_precalc_(/* INPUTS */
   free_big_array_2D_d(N_COSM, a_D_linear);
   free_big_array_2D_d(N_COSM, a_D_median);
   free_big_array_2D_d(N_COSM, a_D_global);
+  free_big_array_2D_d(N_COSM, a_background);
   free_big_array_2D_d(N_COSM, t_background);
 
   free_big_array_3D_d(N_COSM,n_subdom,
