@@ -78,11 +78,21 @@ u[niform])"
 #define INHOM_A_D_DOT_INITIAL_CURV_FACTOR 1
 #endif
 
-/* defined: Hamilton integrator; undefined: Raychaudhuri integrator */
-/*
+/*! defined: Hamilton integrator; undefined: Raychaudhuri integrator */
+/*!
 #define INHOM_ENABLE_HAMILTON_INTEGRATOR 1
 */
 
+/*! If INHOM_TURNAROUND_IS_COLLAPSE is defined, then the moment that a
+   domain reaches turnaround, i.e. when \a \dot{a_D} drops to zero,
+   will be treated as the moment of "collapse". This terminology is
+   only intended for internal use in this program; proposals of better
+   terminology are welcome. :) Obviously, turnaround is \b not really
+   collapse.
+*/
+/*
+#define INHOM_TURNAROUND_IS_COLLAPSE 1
+*/
 
 
 /* for invariants I, II, III */
@@ -264,7 +274,7 @@ double window_R1_R2_k(double R_domain_1,    /* INPUTS: */
                       double k_mod
                       );
 
-#define DIM_BKS00 3
+#define DIM_BKS00 ((size_t)3)
 
 /* k integration bounds @z=200 (?)
    octave: 2*pi ./ [0.0002 0.02 5 50]
@@ -413,6 +423,13 @@ double a_dot_flatFLRW(/* INPUTS: */
                       /* OUTPUTS: */
                       );
 
+double a_ddot_flatFLRW(/* INPUTS: */
+                      struct background_cosm_params_s * background_cosm_params,
+                      double t_background,
+                      int want_verbose
+                      /* OUTPUTS: */
+                      );
+
 
 /* See (49) of Buchert et al RZA2 preprint v20120722 for the formula used
    here. */
@@ -619,12 +636,16 @@ int Omega_D(/* INPUTS: */
             );
 
 
-/* to be called from fortran; underscore convention *not* checked automatically */
-void omega_d_precalc_(/* INPUTS */
-                      int64_t *scal_av_n_gridsize,
+/* to be called from fortran with: bind(C,name='Omega_D_precalc') */
+void Omega_D_precalc(/* INPUTS */
+                      int64_t *scal_av_n_gridsize1,
+                      int64_t *scal_av_n_gridsize2,
+                      int64_t *scal_av_n_gridsize3,
                       double **I_inv,
                       double **II_inv,
                       double **III_inv,
+                      /* aD_norm: proportional to initial volumes^(1/3) */
+                      double **aD_norm,
                       int64_t *n_t_inhomog,
                       double *a_FLRW_init,
                       double *H1bg,
@@ -663,3 +684,29 @@ double ddot_growth_FLRW(/* INPUTS: */
                         );
 
 double print_benchmark(clock_t b0, clock_t b1);
+
+
+/* ***********************************************************
+   Prototypes for deprecated routines
+
+ * ***********************************************************
+*/
+
+/* To be called from fortran provided that the underscore _ convention
+   matches; this is only weakly portable. This old version does not
+   allow unequal scal_av_n_gridsize(1|2|3) values, and does not
+   include normalisation of the initial domains in the case that they
+   are of unequal volumes. So use of Omega_D_precalc() is strongly
+   preferred. */
+void omega_d_precalc_(/* INPUTS */
+                      int64_t *scal_av_n_gridsize1,
+                      double **I_inv,
+                      double **II_inv,
+                      double **III_inv,
+                      int64_t *n_t_inhomog,
+                      double *a_FLRW_init,
+                      double *H1bg,
+                      /* OUTPUTS */
+                      double **t_RZA,
+                      double **a_D_RZA
+                      );
