@@ -293,7 +293,7 @@ double power_spectrum_flatspace_BBKS(double k_mod, /* INPUTS */
         (a_scale_factor_initial*a_scale_factor_initial)/
         (background_cosm_params.inhomog_a_scale_factor_now*background_cosm_params.inhomog_a_scale_factor_now);
 
-      if(background_cosm_params.correct_Pk_norm_known){
+      if(1==background_cosm_params.correct_Pk_norm_known){
         A_all_factors *= background_cosm_params.correct_Pk_norm;
       };
 
@@ -404,7 +404,7 @@ double power_spectrum_flatspace_EisHu_short
     (a_scale_factor_initial*a_scale_factor_initial)/
     (background_cosm_params.inhomog_a_scale_factor_now*background_cosm_params.inhomog_a_scale_factor_now);
 
-  if(background_cosm_params.correct_Pk_norm_known){
+  if(1==background_cosm_params.correct_Pk_norm_known){
     A_all_factors *= background_cosm_params.correct_Pk_norm;
   };
 
@@ -643,7 +643,7 @@ double power_spectrum_flatspace_EisHu_full
     (a_scale_factor_initial*a_scale_factor_initial)/
     (background_cosm_params.inhomog_a_scale_factor_now*background_cosm_params.inhomog_a_scale_factor_now);
 
-  if(background_cosm_params.correct_Pk_norm_known){
+  if(1==background_cosm_params.correct_Pk_norm_known){
     A_all_factors *= background_cosm_params.correct_Pk_norm;
   };
 
