@@ -48,12 +48,12 @@
 #undef DEBUG
 
 /* isolate Q and/or Lambda effect in integrating ODE */
-#ifndef DISABLE_Q
-#define DISABLE_Q 0
-#endif
-#ifndef DISABLE_LAMBDA_IN_ODE
-#define DISABLE_LAMBDA_IN_ODE 0
-#endif
+/*
+DISABLE_Q disables adding Q in the integration of the ODE;
+DISABLE_LAMBDA_IN_ODE disables adding Lambda in the integration of the
+ODE (without modifying other roles of Lambda such as in the
+FLRW a(t), \dot{a}(t) relations).
+*/
 
 /*! \brief Stores first derivatives for the ODE system routine.
  *
@@ -748,6 +748,10 @@ int scale_factor_D_Ham(/* INPUTS: */
   double t_background[N_T_SPLINE];
   int i_t;
 
+#ifndef DISABLE_LAMBDA_IN_ODE
+  double minus_two_Lambda_invGyr2; /* derived parameter: -2 * Lambda in units of Gyr^{-2}; to be calculated from OmLam_0 and H_0 */
+#endif
+
   /* structure of constant and spline tables for integration */
   struct a_D_integrator_params_s a_D_integrator_params;
   double inv_I, inv_I_err;
@@ -861,25 +865,30 @@ int scale_factor_D_Ham(/* INPUTS: */
     for(i_t=0; i_t<N_T_SPLINE; i_t++){
       rza_Q_D_minus_twoLambda_FLRW[i_t] = 0.0;
 
-      if(!DISABLE_Q){
-        rza_Q_D_minus_twoLambda_FLRW[i_t] += rza_Q_D[i_t];
-      };
+#ifndef DISABLE_Q
+      rza_Q_D_minus_twoLambda_FLRW[i_t] += rza_Q_D[i_t];
+#endif
     };
   }else if(1 == background_cosm_params.flatFLRW &&
            background_cosm_params.Omm_0 < 1.0){
+#ifndef DISABLE_LAMBDA_IN_ODE
+    minus_two_Lambda_invGyr2 =
+      -6.0 * background_cosm_params.OmLam_0 *
+      background_cosm_params.H_0 * COSM_H_0_INV_GYR *
+      background_cosm_params.H_0 * COSM_H_0_INV_GYR ;
+#endif
+
     for(i_t=0; i_t<N_T_SPLINE; i_t++){
       rza_Q_D_minus_twoLambda_FLRW[i_t] = 0.0;
 
-      if(!DISABLE_Q){
-        rza_Q_D_minus_twoLambda_FLRW[i_t] += rza_Q_D[i_t];
-      };
+#ifndef DISABLE_Q
+      rza_Q_D_minus_twoLambda_FLRW[i_t] += rza_Q_D[i_t];
+#endif
 
-      if(!DISABLE_LAMBDA_IN_ODE){
-        rza_Q_D_minus_twoLambda_FLRW[i_t] +=
-          -6.0 * background_cosm_params.OmLam_0 *
-          background_cosm_params.H_0 * COSM_H_0_INV_GYR *
-          background_cosm_params.H_0 * COSM_H_0_INV_GYR ;
-      };
+#ifndef DISABLE_LAMBDA_IN_ODE
+      rza_Q_D_minus_twoLambda_FLRW[i_t] +=
+        minus_two_Lambda_invGyr2;
+#endif
     };
   }; /*   if(1 == background_cosm_params.EdS) */
 
