@@ -35,6 +35,8 @@
 #include <malloc.h>
 #endif
 
+#include <time.h>
+
 #include "lib/inhomog.h"
 
 /* int test_Omega_D(
@@ -151,6 +153,18 @@ int main(void)
       {{ 0.07, 0.08, 1.0, 0.99 },
        { 0.01, 0.08, 0.8, 0.99 }} };
 
+  /* TEST_PRECALC is for testing Omega_D with precalculated values of the initial
+     invariants. */
+  /* #define TEST_PRECALC 1 */
+  /* #define BENCHMARK_LCDM 1 */
+
+#ifdef TEST_PRECALC
+  double invs_initial[3]={0.01,1e-4,1e-6};
+#endif
+
+  /* benchmarking */
+  clock_t  benchmark[10]; /* num elements hardwired! */
+  int  i_bench=0;
 
   /* int    I_invariant; */
 
@@ -178,10 +192,17 @@ int main(void)
   /* 'B' = BBKS (BKS version); 'e' = EisHu98 short formula set */
   rza_integrand_params.pow_spec_type = 'e';
 
+#ifndef TEST_PRECALC
   rza_integrand_params.precalculated_invariants.enabled = 0;
   rza_integrand_params.sigma_sq_inv_triple.I_known = 0;
   rza_integrand_params.sigma_sq_inv_triple.II_known = 0;
   rza_integrand_params.sigma_sq_inv_triple.III_known = 0;
+#else
+  rza_integrand_params.precalculated_invariants.enabled = 1;
+  rza_integrand_params.sigma_sq_inv_triple.I_known = invs_initial[0];
+  rza_integrand_params.sigma_sq_inv_triple.II_known = invs_initial[1];
+  rza_integrand_params.sigma_sq_inv_triple.III_known = invs_initial[2];
+#endif
 
 
   /* possible overrides: this may make   expected_a_D_Omegas  cause failure */
@@ -228,11 +249,18 @@ int main(void)
     n_sigma[1]= n_sigma_RZA2[i_RZA2][1];
     n_sigma[2]= n_sigma_RZA2[i_RZA2][2];
 
+    benchmark[i_bench] = clock(); /* initialise timer */
+
     for(i_EdS=0; i_EdS<N_COSM; i_EdS++){
       background_cosm_params.EdS = i_EdS;
       printf("background_cosm_params.EdS  = %d\n",
              background_cosm_params.EdS);
 
+      i_bench ++;
+      benchmark[i_bench]=clock();
+      printf("beginning of i_EdS = %d loop\n",i_EdS);
+      print_benchmark(benchmark[i_bench-1],benchmark[i_bench]);
+
       background_cosm_params.recalculate_t_0 = 1; /* initially recalculate for this test */
       background_cosm_params.Theta2p7 = 2.726/2.7; /* e.g. arXiv:0911.1955 */
       if(1 == background_cosm_params.EdS){
@@ -246,16 +274,23 @@ int main(void)
                            background_cosm_params.inhomog_a_scale_factor_initial,
                            want_verbose);
       }else if(1 == background_cosm_params.flatFLRW){
+#ifndef BENCHMARK_LCDM
         background_cosm_params.H_0 = 70.0;
         background_cosm_params.OmLam_0 = 0.73;
         background_cosm_params.Ombary_0 = 0.044;
         background_cosm_params.sigma8 = 0.83;
-
         /*
           background_cosm_params.H_0 = 50.0;
           background_cosm_params.OmLam_0 = 0.001;
         */
         background_cosm_params.Omm_0 = 1.0 - background_cosm_params.OmLam_0;
+#else
+        background_cosm_params.H_0 = 50.0;
+        background_cosm_params.Omm_0 = 0.999;
+        background_cosm_params.OmLam_0 = 0.001;
+        background_cosm_params.Ombary_0 = 0.088;
+        background_cosm_params.sigma8 = 0.83;
+#endif
         t_i[i_EdS] = t_flatFLRW(&background_cosm_params,
                                 background_cosm_params.inhomog_a_scale_factor_initial,
                                 want_verbose);
@@ -396,17 +431,20 @@ int main(void)
         }; /* for(i_t=0; i_t<N_PLOT_T; i_t++) */
         i_t = N_PLOT_T-1;
 
-        if( fabs( a_D[i_EdS][i_R][i_t]/ a_FLRW_expected
+        if( !isfinite(a_D[i_EdS][i_R][i_t]) ||
+            fabs( a_D[i_EdS][i_R][i_t]/ a_FLRW_expected
                   - expected_Omegas[i_EdS][i_R][0] ) * 0.5 /
             fabs( a_D[i_EdS][i_R][i_t]/ a_FLRW_expected
                   + expected_Omegas[i_EdS][i_R][0] ) > Omega_tol[i_EdS][i_R][0] )
           pass += 1 * pass_eights;
-        if( fabs( Omm_D[i_EdS][i_R][i_t]
+        if( !isfinite(Omm_D[i_EdS][i_R][i_t]) ||
+            fabs( Omm_D[i_EdS][i_R][i_t]
                   - expected_Omegas[i_EdS][i_R][1] ) * 0.5 /
             fabs( Omm_D[i_EdS][i_R][i_t]
                   + expected_Omegas[i_EdS][i_R][1] ) > Omega_tol[i_EdS][i_R][1] )
           pass +=  2  * pass_eights;
-        if( fabs( OmQ_D[i_EdS][i_R][i_t]
+        if( !isfinite(OmQ_D[i_EdS][i_R][i_t]) ||
+            fabs( OmQ_D[i_EdS][i_R][i_t]
                   - expected_Omegas[i_EdS][i_R][2] ) * 0.5 /
             fabs( OmQ_D[i_EdS][i_R][i_t]
                   + expected_Omegas[i_EdS][i_R][2] ) > Omega_tol[i_EdS][i_R][2] )
@@ -434,8 +472,24 @@ int main(void)
                        + expected_Omegas[i_EdS][i_R][2] ));
       }; /* for(i_R=0; i_R<N_PLOT_R; i_R++) */
 
+      i_bench ++;
+      benchmark[i_bench]=clock();
+      printf("...this test took: ");
+      print_benchmark(benchmark[i_bench-1],benchmark[i_bench]);
+
     }; /*   for(i_EdS=0; i_EdS<N_COSM; i_EdS++) */
 
+  i_bench ++;
+  benchmark[i_bench]=clock();
+  printf("...the last few lines took: ");
+  print_benchmark(benchmark[i_bench-1],benchmark[i_bench]);
+
+  printf("tcalc_LCDM/tcalc_EdS: %8ld %7.2f \n",
+         n_calls_invariants,
+         (double)(benchmark[2]-benchmark[1])/
+         (double)(benchmark[4]-benchmark[3]));
+
+
 
     for(i_R=0; i_R<N_PLOT_R; i_R++){
       Omm0 = Omm_D[0][i_R][N_PLOT_T-1];
